/*
 * linux/arch/m68k/atari/svgafb.c -- Low level implementation of the
 *                                   Super VGA frame buffer device
 *
 *    Copyright (C) 1997 Bernd Harries
 *
 *
 * This file is based on the Amiga frame buffer device (amifb.c):
 *
 *    Copyright (C) 1995 Geert Uytterhoeven
 *
 *
 * History:
 *   - 22 Dec 95: Original version by Martin Apel
 *   - 05 Jan 96: Geert: integration into the current source tree
 *
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive
 * for more details.
 */

/*============================================================================*/
/* Includes */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/malloc.h>
#include <linux/delay.h>
#include <linux/zorro.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/irq.h>
#include <asm/pgtable.h>

#include "svgafb.h"

/*============================================================================*/
/* Defines */

#define SVGA_MAX 8
#define arraysize(x)    (sizeof(x)/sizeof(*(x)))

#define  FuncId       if(svgafb_debug & 0x00000100) printk
#define  FuncDone     if(svgafb_debug & 0x00000200) printk
#define  CFuncId      if(svgafb_debug & 0x00000400) printk
#define  CFuncDone    if(svgafb_debug & 0x00000800) printk
#define  OFuncId      if(svgafb_debug & 0x00001000) printk
#define  OFuncDone    if(svgafb_debug & 0x00002000) printk


/* Busy */
#define  Bprint       if(svgafb_debug & 0x01000000) printk
/* Debug */
#define  Dprint       if(svgafb_debug & 0x00800000) printk

/* Message */
#define  Mprint       if(svgafb_debug & 0x00004000) printk
/* Info */
#define  Iprint       if(svgafb_debug & 0x00008000) printk
/* Warn */
#define  Wprint       if(svgafb_debug & 0x00010000) printk
/* Error */
#define  Eprint       if(svgafb_debug & 0x00020000) printk
/* Fatal */
#define  Fprint       if(svgafb_debug & 0x00040000) printk
/* Panic */
#define  Pprint       if(svgafb_debug & 0x00040000) printk


#define OUTB(port,val) \
  *((unsigned volatile char *) ((port)+virt_vgaiobase[vga_num]))=(val)

#define INB(port) \
  (*((unsigned volatile char *) ((port)+virt_vgaiobase[vga_num])))

#define DACDelay                    \
  do {                              \
    unsigned char tmp = INB(0x3da); \
    tmp = INB(0x3da);               \
  } while (0)


/*============================================================================*/
/* Types */

struct svga_fb_par_t
{
   int xres;
   int yres;
   int bpp;
};

/*============================================================================*/
/* Extern Functions*/



/*============================================================================*/
/* Intern Functions*/

/*--------------------------------------------------------------------------*/
   /*
    *    Interface to the low level console driver exported with fb_info
    */

static int  SVGAfb_switch(int con);
static int  SVGAfb_updatevar(int con);
static void SVGAfb_blank(int blank);
static int  SVGAfb_setcmap(struct fb_cmap * cmap, int con);

/*--------------------------------------------------------------------------*/
   /*
    *    Interface fb_ops used by the world exported with fb_info.fb_ops
    */

static int M64_op_open(int fbidx);
static int M64_op_release(int fbidx);
static int M64_op_get_fix(struct fb_fix_screeninfo * fix, int con);
static int M64_op_get_var(struct fb_var_screeninfo * var, int con);
static int M64_op_set_var(struct fb_var_screeninfo * var, int con);
static int M64_op_get_cmap(struct fb_cmap * cmap, int kspc, int con);
static int M64_op_set_cmap(struct fb_cmap * cmap, int kspc, int con);
static int M64_op_pan_display(struct fb_var_screeninfo * var, int con);

static int 
M64_op_ioctl(
  struct inode * inode,
  struct file *file,
  u_int cmd,
  u_long arg,
  int con);

/*--------------------------------------------------------------------------*/
   /*
    *   Internal Routines
    */

static int 
do_fb_set_var(struct fb_var_screeninfo * var, int isactive);

static void 
do_install_cmap(int con);

static void 
Svga64_fb_set_disp(int con);

static int 
get_video_mode(const char * name);

static int 
store_video_par(char* videopar, int num);

static char * 
strtoke(char * s,const char * ct);

/*--------------------------------------------------------------------------*/
   /*
    *   Hardware Specific Routines
    */

static int 
M64_hw_detect(unsigned char vga_num);

static int 
M64_hw_init(unsigned char vga_num);

static int 
M64_hw_encode_fix(struct fb_fix_screeninfo * fix, struct svga_fb_par_t * par);

static int 
M64_hw_decode_var(struct fb_var_screeninfo * var, struct svga_fb_par_t * par);

static int 
M64_hw_encode_var(struct fb_var_screeninfo * var, struct svga_fb_par_t * par);

static void 
M64_hw_get_par(struct svga_fb_par_t * par);

static void 
M64_hw_set_par(struct msvga_b_par * par);

static int 
M64_hw_getcolreg(
  u_int regno,
  u_int * red,
  u_int * green,
  u_int * blue,
  u_int * transp);

static int 
M64_hw_setcolreg(u_int regno, u_int red, u_int green, u_int blue, u_int transp);

static void 
M64_hw_blank(int blank);



/*============================================================================*/
/*============================================================================*/
/* Intern Variables */

/*--------------------------------------------------------------------------*/
/* BSS */


static unsigned long            phys_vmembase[SVGA_MAX];
static unsigned long            phys_size[SVGA_MAX];
static unsigned long            phys_vgaiobase[SVGA_MAX];

static unsigned long            virt_vmembase[SVGA_MAX];
static unsigned long            virt_size[SVGA_MAX];
static unsigned long            virt_vgaiobase[SVGA_MAX];

static unsigned int             svga_xres[SVGA_MAX];
static unsigned int             svga_yres[SVGA_MAX];
static unsigned int             svga_depth[SVGA_MAX];


static struct svga_fb_par_t     current_par[SVGA_MAX];
static struct display           vga_disp[SVGA_MAX];
static struct fb_info           fb_info[SVGA_MAX];

static u_short                  svga_colormap [SVGA_MAX][256][4];

static long *                   memstart;

static u_int                    svga_key;

static int                      currcon;


static u_short                  svga_inverse[SVGA_MAX];
static short int                svga_pmode[SVGA_MAX];
static short int                svga_mode[SVGA_MAX];   /* 0..31 */
static char                     current_par_valid[SVGA_MAX];

static unsigned char            svga_count;


/*--------------------------------------------------------------------------*/
   /*
    *    Switch for Chipset Independency
    */

static struct fb_hwswitch
{

  /* Initialisation */

  int (*init)(unsigned char num);

  /* Display Control */

  int (*encode_fix)(struct fb_fix_screeninfo * fix, struct svga_fb_par_t * par);
  int (*decode_var)(struct fb_var_screeninfo * var, struct svga_fb_par_t * par);
  int (*encode_var)(struct fb_var_screeninfo * var, struct svga_fb_par_t * par);
  void (*get_par)(struct svga_fb_par_t * par );
  void (*set_par)(struct svga_fb_par_t * par );
  int (*getcolreg)(u_int regno, u_int * red, u_int * green, u_int * blue,
    u_int * transp);
  int (*setcolreg)(u_int regno, u_int red, u_int green, u_int blue,
    u_int transp);
  void (*blank)(int blank);
  void (*set_screen_base)( unsigned long s_base );
  int  (*pan_display)( struct fb_var_screeninfo * var);
} 
* fbhws_ptr;


/*--------------------------------------------------------------------------*/
/* Data */


/* -------------- local Interfaces to hardware functions ---------------- */


static struct fb_hwswitch 
svga_switch = 
{
  M64_hw_detect,
  M64_hw_encode_fix,
  M64_hw_decode_var,
  M64_hw_encode_var,
  NULL,                    /* M64_hw_get_par, */
  NULL,                    /* M64_hw_set_par, */
  M64_hw_getcolreg,
  M64_hw_setcolreg,
  M64_hw_blank,
  NULL,
  NULL
};

/* ------ Interfaces to hardware functions to export with fb_info  ------ */

static struct fb_ops 
svga_fb_ops =
{
  M64_op_open,
  M64_op_release,
  M64_op_get_fix,
  M64_op_get_var,
  M64_op_set_var,
  M64_op_get_cmap,
  M64_op_set_cmap,
  M64_op_pan_display,
  M64_op_ioctl
};

static  unsigned int    svgafb_debug = 0x018FCF00;


static  unsigned int    svga_bitspercol[SVGA_MAX] = {8, 8, 8, 8};

static int      svga_fb_8 = 1;        /* Use S VGA board */

   /*
    *    Frame Buffer Name
    */

static char     svga_fb_name[16] = "Super VGA";


   /*
    *    Super VGA Graphics Board
    */

#define M64_8_WIDTH 1152
#define M64_8_HEIGHT 912
#define M64_8_PIXCLOCK 12500    /* ++Geert: Just a guess */

#define M64_16_WIDTH 800
#define M64_16_HEIGHT 600
#define M64_16_PIXCLOCK 25000   /* ++Geert: Just a guess */


/*--------------------------------------------------------------------------*/
   /*
    *    Predefined Video Mode Names
    */
/*--------------------------------------------------------------------------*/

static char *                   svga_fb_modenames[] =
{

   /*
    *    Autodetect (Default) Video Mode
    */

   "default",

   /*
    *    Predefined Video Modes
    */
    
   "SVGA_8",            /* Svga 8 bpp */
   "SVGA_16",           /* Svga 16 bpp */

   /*
    *    Dummy Video Modes
    */

   "dummy", "dummy", "dummy", "dummy", "dummy", "dummy", "dummy", "dummy",
   "dummy", "dummy", "dummy", "dummy", "dummy", "dummy", "dummy", "dummy",
   "dummy", "dummy", "dummy", "dummy",

   /*
    *    User Defined Video Modes
    *
    *    This doesn't work yet!!
    */

   "user0", "user1", "user2", "user3", "user4", "user5", "user6", "user7"
};


/*--------------------------------------------------------------------------*/
   /*
    *    Predefined Video Mode Definitions
    */
/*--------------------------------------------------------------------------*/

static struct fb_var_screeninfo svga_fb_predefined[] = 
{

   /*
    *    Autodetect (Default) Video Mode
    */

   { 0, },

   /*
    *    Predefined Video Modes
    */
    
   {
      /* Cybervision 8 bpp */
      M64_8_WIDTH, M64_8_HEIGHT, M64_8_WIDTH, M64_8_HEIGHT, 0, 0, 8, 0,
      {0, 8, 0}, {0, 8, 0}, {0, 8, 0}, {0, 0, 0},
      0, 0, -1, -1, FB_ACCEL_NONE, M64_8_PIXCLOCK, 64, 96, 35, 12, 112, 2,
      FB_SYNC_COMP_HIGH_ACT|FB_SYNC_VERT_HIGH_ACT, FB_VMODE_NONINTERLACED
   }, {
      /* Cybervision 16 bpp */
      M64_16_WIDTH, M64_16_HEIGHT, M64_16_WIDTH, M64_16_HEIGHT, 0, 0, 16, 0,
      {11, 5, 0}, {5, 6, 0}, {0, 5, 0}, {0, 0, 0},
      0, 0, -1, -1, FB_ACCEL_NONE, M64_16_PIXCLOCK, 64, 96, 35, 12, 112, 2,
      FB_SYNC_COMP_HIGH_ACT|FB_SYNC_VERT_HIGH_ACT, FB_VMODE_NONINTERLACED
   },

   /*
    *    Dummy Video Modes
    */

   { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, },
   { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, },
   { 0, }, { 0, },

   /*
    *    User Defined Video Modes
    */

   { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }, { 0, }
};


#define NUM_TOTAL_MODES    arraysize(svga_fb_predefined)
#define NUM_PREDEF_MODES   (3)


   /*
    *    Some default modes
    */

#define M64_8_DEFMODE     (1)
#define M64_16_DEFMODE    (2)



/*============================================================================*/
/*============================================================================*/
/* Functions */

/*============================================================================*/
/*============================================================================*/


/*--------------------------------------------------------------------------*/

static int 
M64_op_open(int fbidx)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  OFuncId("M64_op_open(%02d) \n", fbidx);
  /*
   * Nothing, only a usage count for the moment
   */

  MOD_INC_USE_COUNT;
  return(0);
}
/*endproc M64_op_open() */

/*============================================================================*/

/*--------------------------------------------------------------------------*/

static int 
M64_op_release(int fbidx)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  FuncId("M64_op_release() \n");
  MOD_DEC_USE_COUNT;
  return(0);
}
/*endproc M64_op_release() */

/*============================================================================*/


/* -------------------- Hardware specific routines ------------------------ */


/*--------------------------------------------------------------------------*/

static int 
M64_hw_detect(unsigned char vga_num)

/*--------------------------------------------------------------------------*/
{
  struct fb_var_screeninfo *    var_ptr = &svga_fb_predefined[0];
  struct svga_fb_par_t          dummy_par;

  /*--------------------------------------------------------------------------*/
  FuncId("M64_hw_detect() \n");

  dummy_par.xres = svga_xres[vga_num];
  dummy_par.yres = svga_yres[vga_num];
  dummy_par.bpp = svga_depth[vga_num];
  M64_hw_encode_var(var_ptr, &dummy_par);

  return(1);    /* detected_mode */
}
/*endproc M64_hw_init() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Initialization
    *
    *    Set the default video mode for this chipset. If a video mode was
    *    specified on the command line, it will override the default mode.
    */
/*--------------------------------------------------------------------------*/

static int 
M64_hw_init(unsigned char vga_num)

/*--------------------------------------------------------------------------*/
{
  int                           i;
  char size;
  volatile u_long *             CursorBase;
  unsigned long                 board_addr;
  /*
  const struct                  ConfigDev *cd;
  */
  /*--------------------------------------------------------------------------*/
  FuncId("M64_hw_init() \n");

  if(svga_mode[vga_num] == -1)
  {
    svga_mode[vga_num] = M64_8_DEFMODE;
  }
  /*endif*/

  board_addr = virt_vmembase[vga_num];
  for (i = 0; i < 256; i++)
  {
    svga_colormap [vga_num][i][0] = i;
    svga_colormap [vga_num][i][1] = i;
    svga_colormap [vga_num][i][2] = i;
    svga_colormap [vga_num][i][3] = 0;
  }


  M64_hw_setcolreg (255, 56, 100, 160, 0);
  M64_hw_setcolreg (254, 0, 0, 0, 0);

  FuncDone("M64_hw_init() done.\n");
  return(0);
}
/*endproc M64_hw_init() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    This function should fill in the `fix' structure based on the
    *    values in the `par' structure.
    */
/*--------------------------------------------------------------------------*/

static int 
M64_hw_encode_fix(struct fb_fix_screeninfo * fix, struct svga_fb_par_t * par)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;
  int                           i;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_hw_encode_fix() \n");

  strcpy(fix->id, svga_fb_name);
  fix->smem_start = virt_vmembase[vga_num];

  /* aufrunden */
  fix->smem_len = (phys_size[vga_num] + PAGE_SIZE -1) & PAGE_MASK;

  if(svga_depth[vga_num] == 1)
  {
    fix->type = FB_TYPE_PACKED_PIXELS;
    /* The letters 'n' and 'i' in the "video=SVGA:" stand
     * for "normal" and "inverted", rsp., in the monochrome case */
    fix->visual =
      (svga_pmode[vga_num] == FB_TYPE_INTERLEAVED_PLANES ||
      svga_pmode[vga_num] == FB_TYPE_PACKED_PIXELS) ?
      FB_VISUAL_MONO10 : FB_VISUAL_MONO01;
  }
  else   /* depth > 1 */
  {
    switch(svga_pmode[vga_num])
    {
      /* All visuals are STATIC, because we don't know how to change
       * colors :-(
       */
      case -1:              /* truecolor */
        fix->type = FB_TYPE_PACKED_PIXELS;
        fix->visual = FB_VISUAL_TRUECOLOR;
        break;
      case FB_TYPE_PACKED_PIXELS:
        fix->type = FB_TYPE_PACKED_PIXELS;
        fix->visual = FB_VISUAL_STATIC_PSEUDOCOLOR;
        break;
      case FB_TYPE_PLANES:
        fix->type = FB_TYPE_PLANES;
        fix->visual = FB_VISUAL_STATIC_PSEUDOCOLOR;
        break;
      case FB_TYPE_INTERLEAVED_PLANES:
        fix->type = FB_TYPE_INTERLEAVED_PLANES;
        fix->type_aux = 2;
        fix->visual = FB_VISUAL_STATIC_PSEUDOCOLOR;
        break;
    }
    /*endswitch*/
  }
  /*endif*/
  fix->xpanstep = 0;
  fix->ypanstep = 0;
  fix->ywrapstep = 0;
  fix->line_length = 0;

  for(i = 0; i < arraysize(fix->reserved); i++)
    fix->reserved[i] = 0;

  return(0);
}
/*endproc _encode_fix() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Get the video params out of `var'. If a value doesn't fit, round
    *    it up, if it's too big, return -EINVAL.
    */
/*--------------------------------------------------------------------------*/

static int 
M64_hw_decode_var(struct fb_var_screeninfo * var, struct svga_fb_par_t * par)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_hw_decode_var() \n");

  par->xres = var->xres;
  par->yres = var->yres;
  par->bpp = var->bits_per_pixel;

  return(0);
}
/*endproc _decode_var() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Fill the `var' structure based on the values in `par' and maybe
    *    other values read out of the hardware.
    */
/*--------------------------------------------------------------------------*/

static int 
M64_hw_encode_var(struct fb_var_screeninfo * var, struct svga_fb_par_t * par)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;
  int                           i;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_hw_encode_var() \n");

  var->xres = par->xres;
  var->yres = par->yres;
  var->xres_virtual = par->xres;
  var->yres_virtual = par->yres;
  var->xoffset = 0;
  var->yoffset = 0;

  var->bits_per_pixel = par->bpp;
  var->grayscale = 0;

  if(par->bpp == 8)
  {
    var->red.offset = 0;
    var->red.length =
      (virt_vgaiobase[vga_num] ? svga_bitspercol[vga_num]: 0);
    var->red.msb_right = 0;
    var->blue = var->green = var->red;
  }
  else
  {
    var->red.offset = 11;
    var->red.length = 5;
    var->red.msb_right = 0;
    var->green.offset = 5;
    var->green.length = 6;
    var->green.msb_right = 0;
    var->blue.offset = 0;
    var->blue.length = 5;
    var->blue.msb_right = 0;
  }
  var->transp.offset = 0;
  var->transp.length = 0;
  var->transp.msb_right = 0;

  var->nonstd = 0;
  var->activate = 0;

  var->height = -1;
  var->width = -1;
  var->accel = FB_ACCEL_NONE;
  var->vmode = FB_VMODE_NONINTERLACED;

  /* Dummy values */

  if (par->bpp == 8)
    var->pixclock = M64_8_PIXCLOCK;
  else
    var->pixclock = M64_16_PIXCLOCK;
  var->sync = 0;
  var->left_margin = 64;
  var->right_margin = 96;
  var->upper_margin = 35;
  var->lower_margin = 12;
  var->hsync_len = 112;
  var->vsync_len = 2;

  for(i = 0; i < arraysize(var->reserved); i++)
    var->reserved[i] = 0;

  return(0);
}
/*endproc _encode_var() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Set a single color register. The values supplied are already
    *    rounded down to the hardware's capabilities (according to the
    *    entries in the var structure). Return != 0 for invalid regno.
    */
/*--------------------------------------------------------------------------*/

static int 
M64_hw_setcolreg(u_int regno,
  u_int red,
  u_int green,
  u_int blue,
  u_int transp)

/*--------------------------------------------------------------------------*/
{
  static short                  vga_num;

  unsigned short                colmask;
  unsigned short                colshift;

  /*--------------------------------------------------------------------------*/
  OFuncId("M64_hw_setcolreg(%02X) \n", regno);

  if(regno > 255)  return (1);
  svga_colormap [vga_num][regno][0] = red;
  svga_colormap [vga_num][regno][1] = green;
  svga_colormap [vga_num][regno][2] = blue;
  svga_colormap [vga_num][regno][3] = transp;

  if(! phys_vgaiobase)  return 1;

  colmask = (1 << svga_bitspercol[vga_num]) - 1;
  colshift = 8 - svga_bitspercol[vga_num];

  OUTB(0x3c8, regno);
  DACDelay;
  OUTB(0x3c9, (red >> colshift) & colmask);
  DACDelay;
  OUTB(0x3c9, (green >> colshift) & colmask);
  DACDelay;
  OUTB(0x3c9, (blue >> colshift) & colmask);
  DACDelay;

  return (0);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Read a single color register and split it into
    *    colors/transparent. Return != 0 for invalid regno.
    */
/*--------------------------------------------------------------------------*/

static int 
M64_hw_getcolreg(u_int regno,
  u_int * red,
  u_int * green,
  u_int * blue,
  u_int * transp)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;

  /*--------------------------------------------------------------------------*/
  OFuncId("M64_hw_getcolreg() \n");

  if(regno >= 256)  return (1);

  *red    = svga_colormap [vga_num][regno][0];
  *green  = svga_colormap [vga_num][regno][1];
  *blue   = svga_colormap [vga_num][regno][2];
  *transp = svga_colormap [vga_num][regno][3];

  /*
  OUTB(0x3c7, regno);
  DACDelay;
  *red = INB(0x3c9) & colmask;
  DACDelay;
  *green = INB(0x3c9) & colmask;
  DACDelay;
  *blue = INB(0x3c9) & colmask;
  DACDelay;
  */
	    
  return (0);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    (Un)Blank the screen: Set all colors = RGB 000 or back
    */
/*--------------------------------------------------------------------------*/

void 
M64_hw_blank(int blank)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;
  int regno;

  unsigned short                colmask;
  unsigned short                colshift;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_hw_blank() \n");

  if(blank)
  {
    for(regno = 0; regno < 256; regno++)
    {
      OUTB(0x3c8, regno);
      DACDelay;
      OUTB(0x3c9, 0);
      DACDelay;
      OUTB(0x3c9, 0);
      DACDelay;
      OUTB(0x3c9, 0);
      DACDelay;

    }
    /*nexxt regno*/
  }
  else
  {
    colmask = (1 << svga_bitspercol[vga_num]) - 1;
    colshift = 8 - svga_bitspercol[vga_num];
    for (regno = 0; regno < 256; regno++)
    {
      OUTB(0x3c8, regno);
      DACDelay;
      OUTB(0x3c9, (svga_colormap [vga_num][regno][0] >> colshift) & colmask);
      DACDelay;
      OUTB(0x3c9, (svga_colormap [vga_num][regno][1] >> colshift) & colmask);
      DACDelay;
      OUTB(0x3c9, (svga_colormap [vga_num][regno][2] >> colshift) & colmask);
    }
    /*nexxt regno*/
  }
  /*endif*/

  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
/*
 * We are waiting for "fifo" FIFO-slots empty
 */
/*--------------------------------------------------------------------------*/

void 
SVGA_WaitQueue (u_short fifo)

/*--------------------------------------------------------------------------*/
{
  u_short                status;

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGA_WaitQueue() \n");


  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
/*
 * We are waiting for Hardware (Graphics Engine) not busy
 */
/*--------------------------------------------------------------------------*/

void 
SVGA_WaitBlit (void)

/*--------------------------------------------------------------------------*/
{
  u_short          status;

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGA_WaitBlit() \n");

  return;


  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
/*
 * BitBLT - Through the Plane
 */
/*--------------------------------------------------------------------------*/

void 
SVGA_BitBLT (
  u_short curx,
  u_short cury,
  u_short destx,
  u_short desty,
  u_short width,
  u_short height,
  u_short mode)

/*--------------------------------------------------------------------------*/
{
  /* u_short blitcmd = S3_BITBLT; */

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGA_BitBlt() \n");
  /* Set drawing direction */
  /* -Y, X maj, -X (default) */

  SVGA_WaitQueue (0x8000);

  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
/*
 * Rectangle Fill Solid
 */
/*--------------------------------------------------------------------------*/

void 
SVGA_RectFill (
  u_short x,
  u_short y,
  u_short width,
  u_short height,
  u_short mode,
  u_short color)

/*--------------------------------------------------------------------------*/
{
  /* u_short blitcmd = S3_FILLEDRECT; */

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGA_RectFill() \n");

  SVGA_WaitQueue (0x8000);

  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
/*
 * Move cursor to x, y
 */
/*--------------------------------------------------------------------------*/

void 
SVGA_MoveCursor (u_short x, u_short y)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGA_MoveCursor() \n");


  return;
}
/*endproc SVGA_MoveCursor()*/

/*============================================================================*/


/* -------------------- Generic routines ------------------------------------ */


/*--------------------------------------------------------------------------*/
   /*
    *    Fill the hardware's `par' structure.
    */
/*--------------------------------------------------------------------------*/

static void 
M64_hw_get_par(struct svga_fb_par_t * par)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_hw_get_par() \n");
  if(current_par_valid[vga_num])
  {
    *par = current_par[vga_num];
  }
  else    /* use a default, initialize */
  {
    fbhws_ptr->decode_var(
      &svga_fb_predefined[svga_mode[vga_num]],
      par);
  }
  /*endif*/

  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/

static void 
M64_hw_set_par(struct svga_fb_par_t * par)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_hw_set_par() \n");
  current_par[vga_num] = *par;
  current_par_valid[vga_num] = 1;


  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/

static int 
do_fb_set_var(struct fb_var_screeninfo * var, int isactive)

/*--------------------------------------------------------------------------*/
{
  int                           err, activate;
  struct svga_fb_par_t          par;

  /*--------------------------------------------------------------------------*/
  CFuncId("do_fb_set_var() \n");
  if((err = fbhws_ptr->decode_var(var, &par)))
    return(err);
  activate = var->activate;
  if((var->activate & FB_ACTIVATE_MASK) == FB_ACTIVATE_NOW && isactive)
  {
    M64_hw_set_par(&par);
  }
  /*endif*/
  fbhws_ptr->encode_var(var, &par);
  var->activate = activate;
  return(0);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/

static void 
do_install_cmap(int con)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  CFuncId("do_install_cmap() \n");

  if(con != currcon)  return;

  if(fb_display[con].cmap.len)
  {
    fb_set_cmap(&fb_display[con].cmap,
      &fb_display[con].var,
      1,
      fbhws_ptr->setcolreg);
  }
  else
  {
    fb_set_cmap(fb_default_cmap(fb_display[con].var.bits_per_pixel),
      &fb_display[con].var,
      1,
      fbhws_ptr->setcolreg);
  }
  /*endif*/

  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Get the Fixed Part of the Display
    */
/*--------------------------------------------------------------------------*/

static int 
M64_op_get_fix(struct fb_fix_screeninfo * fix, int con)

/*--------------------------------------------------------------------------*/
{
  struct svga_fb_par_t          par;
  int                           status = 0;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_op_get_fix() \n");
  if (con == -1)
  {
    M64_hw_get_par(&par);
  }
  else
  {
    status = fbhws_ptr->decode_var(&fb_display[con].var, &par);
  }
  /*endif*/
  return(status ? status : fbhws_ptr->encode_fix(fix, &par));
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Get the User Defined Part of the Display
    */
/*--------------------------------------------------------------------------*/

static int 
M64_op_get_var(struct fb_var_screeninfo *var, int con)

/*--------------------------------------------------------------------------*/
{
  struct svga_fb_par_t          par;
  int                           status = 0;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_op_get_var() \n");
  if(con == -1)
  {
    M64_hw_get_par(&par);
    status = fbhws_ptr->encode_var(var, &par);
  }
  else
  {
    *var = fb_display[con].var;
  }
  /*endif*/
  return(status);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/

static void 
Svga_fb_set_disp(int con)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;

  struct display *              disp_p;
  struct fb_fix_screeninfo      fix;

  /*--------------------------------------------------------------------------*/
  CFuncId("Svga_fb_set_disp() \n");

  if(con >= 0)
    disp_p = &fb_display[con];
  else
    disp_p = &vga_disp[vga_num];  	/* used during initialization */

  M64_op_get_fix(&fix, con);

  if(con == -1)  con = 0;

  disp_p->screen_base = (u_char *)fix.smem_start;
  disp_p->visual = fix.visual;
  disp_p->type = fix.type;
  disp_p->type_aux = fix.type_aux;
  disp_p->ypanstep = fix.ypanstep;
  disp_p->ywrapstep = fix.ywrapstep;
  disp_p->line_length = fix.line_length;
  if(fix.visual != FB_VISUAL_PSEUDOCOLOR && fix.visual != FB_VISUAL_DIRECTCOLOR)
  {
    disp_p->can_soft_blank = 0;                   /* aus atari_ */
  }
  else
  {
    disp_p->can_soft_blank = 1;
  }
  /*endif*/

  disp_p->can_soft_blank = 1;                       /* aus Cyber_ */
  disp_p->inverse = svga_inverse[vga_num];

  return;
}
/*endproc Svga_fb_set_disp() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
  /*
   *    Set the User Defined Part of the Display
   */
/*--------------------------------------------------------------------------*/

static int 
M64_op_set_var(struct fb_var_screeninfo * var, int con)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;
  int       err, oldxres, oldyres, oldvxres, oldvyres, oldbpp;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_op_set_var() \n");

  if((err = do_fb_set_var(var, con == currcon)))
    return(err);
  if((var->activate & FB_ACTIVATE_MASK) == FB_ACTIVATE_NOW)
  {
    oldxres = fb_display[con].var.xres;
    oldyres = fb_display[con].var.yres;
    oldvxres = fb_display[con].var.xres_virtual;
    oldvyres = fb_display[con].var.yres_virtual;
    oldbpp = fb_display[con].var.bits_per_pixel;
    fb_display[con].var = *var;
    if(oldxres != var->xres || oldyres != var->yres ||
      oldvxres != var->xres_virtual || oldvyres != var->yres_virtual ||
      oldbpp != var->bits_per_pixel)
    {
      Svga_fb_set_disp(con);
      (*fb_info[vga_num].changevar)(con);
      fb_alloc_cmap(&fb_display[con].cmap, 0, 0);
      do_install_cmap(con);
    }
  }
  /*endif*/
  var->activate = 0;

  return(0);
}
/*endproc M64_op_set_var() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Get the Colormap
    */
/*--------------------------------------------------------------------------*/

static int 
M64_op_get_cmap(struct fb_cmap * cmap, int kspc, int con)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  OFuncId("M64_op_get_cmap() \n");

  if(con == currcon) /* current console? */
  {
    return(fb_get_cmap(cmap, &fb_display[con].var, kspc, fbhws_ptr->getcolreg));
  }
  else if(fb_display[con].cmap.len) /* non default colormap? */
  {
    fb_copy_cmap(&fb_display[con].cmap, cmap, kspc ? 0 : 2);
  }
  else
  {
    fb_copy_cmap(fb_default_cmap(fb_display[con].var.bits_per_pixel),
      cmap, kspc ? 0 : 2);
  }
  /*endif*/

  return(0);
}
/*endproc M64_op_get_cmap() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Set the Colormap
    */
/*--------------------------------------------------------------------------*/

static int 
M64_op_set_cmap(struct fb_cmap * cmap, int kspc, int con)

/*--------------------------------------------------------------------------*/
{
  int err;

  /*--------------------------------------------------------------------------*/
  OFuncId("M64_op_set_cmap() \n");

  if (!fb_display[con].cmap.len)       /* no colormap allocated? */
  {
    err = fb_alloc_cmap(&fb_display[con].cmap,
      1 << fb_display[con].var.bits_per_pixel,
      0);
    if(err)
    {
      return(err);
    }
    /*endif*/
  }
  /*endif*/
  if(con == currcon)              /* current console? */
  {
    return(fb_set_cmap(cmap, &fb_display[con].var, kspc, fbhws_ptr->setcolreg));
  }
  else
  {
    fb_copy_cmap(cmap, &fb_display[con].cmap, kspc ? 0 : 1);
  }
  /*endif*/

  return(0);
}
/*endproc M64_op_set_cmap() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Pan or Wrap the Display
    *
    *    This call looks only at xoffset, yoffset and the FB_VMODE_YWRAP flag
    */
/*--------------------------------------------------------------------------*/

static int 
M64_op_pan_display(struct fb_var_screeninfo *var, int con)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  CFuncId("Svga_pan_display() \n");

  return(-EINVAL);
}
/*endproc M64_op_pan_display() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *   Super VGA Frame Buffer Specific ioctls
    */
/*--------------------------------------------------------------------------*/

static int 
M64_op_ioctl(
  struct inode * inode,
  struct file *  file,
  u_int          cmd,
  u_long         arg,
  int            con)

/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;

  /*--------------------------------------------------------------------------*/
  CFuncId("M64_op_ioctl($%08X %d) \n", cmd, con);
  switch (cmd)
  {

#ifdef FBCMD_GET_CURRENTPAR
    case FBCMD_GET_CURRENTPAR:
    {
      if (copy_to_user((void *)arg, (void *)&current_par[vga_num],
         sizeof(struct svga_fb_par_t)))
        return -EFAULT;
      return 0;
    }
    /*endcase*/
#endif

#ifdef FBCMD_SET_CURRENTPAR
    case FBCMD_SET_CURRENTPAR:
    {

      if (copy_from_user((void *)&current_par[vga_num], (void *)arg,
        sizeof(struct svga_fb_par_t)))
        return -EFAULT;
      M64_hw_set_par(&current_par[vga_num]);
      return 0;
    }
    /*endcase*/
#endif

  }
  /*endswitch*/
  return(-EINVAL);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/

int 
SVGA_probe(void)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  FuncId("SVGA_probe() \n");
  svga_key = 0;
  return(svga_key);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/

__initfunc(
void 
SVGA_video_setup(char * options, int * ints)
)
/*--------------------------------------------------------------------------*/
{
  static unsigned char          vga_num;

  char *        this_opt;
  char          svga_str[80];

  /*--------------------------------------------------------------------------*/
  FuncId("SVGA_video_setup() '%s'\n", options);

  fb_info[vga_num].fontname[0] = '\0';

  if(!options || !*options)  return;

  for(this_opt = strtok(options, ","); this_opt; this_opt = strtok(NULL, ","))
  {
    if (!strcmp(this_opt, "inverse"))
    {
      svga_inverse[vga_num] = 1;
      fb_invert_cmaps();
    }
    else if (!strncmp(this_opt, "font:", 5))
    {
      strcpy(fb_info[vga_num].fontname, this_opt + 5);
    }
    else if (!strncmp(this_opt,"SVGA:", 5))
    {
      strncpy(svga_str, this_opt + 7, 80);
      store_video_par(svga_str, vga_num);
      svga_count = ++vga_num;
    }
    else
    {
      svga_mode[vga_num] = get_video_mode(this_opt);
    }
    /*endif*/
  }
  /*next this_opt*/

  return;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/

static int 
store_video_par(char* video_str, int vga_num)

/*--------------------------------------------------------------------------*/
{
  char *                        p;
  unsigned  long                sm_addr, sm_size;

  int                           xres, yres, depth;
  short int                     pmode;

  /*--------------------------------------------------------------------------*/
  CFuncId("store_video_par(%d) \n", vga_num);
  /* Format is: <xres>;<yres>;<depth>;<plane organ.>;
   *            <screen mem addr>
   *        [;<screen mem length>[;<vgaiobase>[;<colorreg-type>]]]
   */

  if (!(p = strtoke(video_str, ";")) || !*p)  goto svga_invalid;
  xres = simple_strtoul(p, NULL, 10);
  if (xres <= 0)  goto svga_invalid;

  if (!(p = strtoke(NULL, ";")) || !*p)  goto svga_invalid;
  yres = simple_strtoul(p, NULL, 10);
  if (yres <= 0) goto svga_invalid;

  if (!(p = strtoke(NULL, ";")) || !*p)  goto svga_invalid;
  depth = simple_strtoul(p, NULL, 10);
  if (depth != 1 && depth != 2 && depth != 4 && depth != 8 &&
      depth != 16 && depth != 24)  goto svga_invalid;

  if (!(p = strtoke(NULL, ";")) || !*p)  goto svga_invalid;
  if (*p == 'i')
    pmode = FB_TYPE_INTERLEAVED_PLANES;
  else if (*p == 'p')
    pmode = FB_TYPE_PACKED_PIXELS;
  else if (*p == 'n')
    pmode = FB_TYPE_PLANES;
  else if (*p == 't')
    pmode = -1; /* true color */
  else
    goto svga_invalid;


  if (!(p = strtoke(NULL, ";")) ||!*p)  goto svga_invalid;
  sm_addr = simple_strtoul(p, NULL, 0);

  if (!(p = strtoke(NULL, ";")) ||!*p)
    sm_size = xres * yres * depth / 8;
  else
    sm_size = simple_strtoul(p, NULL, 0);

  if ((p = strtoke(NULL, ";")) && *p)
    phys_vgaiobase[vga_num] = simple_strtoul(p, NULL, 0);

  if ((p = strtoke(NULL, ";")) && *p)
  {
    svga_bitspercol[vga_num] = simple_strtoul(p, NULL, 0);
    if (svga_bitspercol[vga_num] > 8)
      svga_bitspercol[vga_num] = 8;
    else if (svga_bitspercol[vga_num] < 1)
      svga_bitspercol[vga_num] = 1;
  }
  /*endif*/
  

  svga_xres[vga_num]  = xres;
  svga_yres[vga_num]  = yres;
  svga_depth[vga_num] = depth;
  svga_pmode[vga_num] = pmode;
  phys_vmembase[vga_num] = sm_addr;
  phys_size[vga_num]  = sm_size;

  return(0);

  /*--------------------------------------------------------------------------*/
  svga_invalid:

  phys_vmembase[vga_num] = 0;

  return(-1);
}
/*endproc*/

/*============================================================================*/


/* a strtok which returns empty strings, too */

/*--------------------------------------------------------------------------*/

static char * 
strtoke(char * s, const char * ct)

/*--------------------------------------------------------------------------*/
{
  static char    *ssave = NULL;
  char          *sbegin, *send;
  
  /*--------------------------------------------------------------------------*/
  sbegin  = s ? s : ssave;
  if (!sbegin) {
    return NULL;
  }
  if (*sbegin == '\0') {
    ssave = NULL;
    return NULL;
  }
  send = strpbrk(sbegin, ct);
  if (send && *send != '\0')
    *send++ = '\0';
  ssave = send;
  return sbegin;
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Initialization
    */
/*--------------------------------------------------------------------------*/

__initfunc(
int
SVGA_fb_init(u_long * mem_start)
)
/*--------------------------------------------------------------------------*/
{
  static char                   fb_init_count;

  struct svga_fb_par_t          par;
  int                           err;
  unsigned char                 vga_num;

  /*--------------------------------------------------------------------------*/
  FuncId("SVGA_fb_init($%08lX) \n", *mem_start);

  vga_num = fb_init_count;
  if(phys_vmembase[vga_num] == 0)  return -ENODEV;

  fb_init_count++;
  memstart = mem_start;

  fbhws_ptr = &svga_switch;

  /* Map the video memory (physical address given) to somewhere
   * in the kernel address space.
   */
  *mem_start = (*mem_start + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1);
  virt_vmembase[vga_num] = kernel_map(
    phys_vmembase[vga_num],
    phys_size[vga_num],
    KERNELMAP_NO_COPYBACK,    /* must be changed if hardware blit */
    mem_start);
  if (phys_vgaiobase[vga_num])
  {
    virt_vgaiobase[vga_num] = kernel_map(
      phys_vgaiobase[vga_num],
      0x10000,                           /* 64 K */
      KERNELMAP_NOCACHE_SER,
      mem_start);
  }
  /*endif*/

  virt_size[vga_num] = phys_size[vga_num] & PAGE_MASK;  /* abrunden */

  memset((char *) virt_vmembase[vga_num], 0, virt_size[vga_num]);

  strcpy(fb_info[vga_num].modename, svga_fb_name);
  fb_info[vga_num].changevar = NULL;
  fb_info[vga_num].node = -1;
  fb_info[vga_num].fbops = &svga_fb_ops;
  fb_info[vga_num].fbvar_num = NUM_TOTAL_MODES;
  fb_info[vga_num].fbvar = svga_fb_predefined;
  fb_info[vga_num].disp = &vga_disp[vga_num];
  fb_info[vga_num].switch_con = &SVGAfb_switch;
  fb_info[vga_num].updatevar = &SVGAfb_updatevar;
  fb_info[vga_num].blank = &SVGAfb_blank;
  fb_info[vga_num].setcmap = &SVGAfb_setcmap;

  err = register_framebuffer(&fb_info[vga_num]);
  if(err < 0)
  {
    Eprint(" Cannot register frame buffer() \n" );
    return(err);
  }
  /*endif*/

  fbhws_ptr->init(vga_num);
  fbhws_ptr->decode_var(&svga_fb_predefined[svga_mode[vga_num]], &par);
  fbhws_ptr->encode_var(&svga_fb_predefined[0], &par); /* store resolution */

  do_fb_set_var(&svga_fb_predefined[0], 1);

  M64_op_get_var(&vga_disp[vga_num].var, -1);
  Svga_fb_set_disp(-1);

  Iprint(" Determined %dx%d, depth %d $%08lX MEM @ $%08lX IO @ $%08lX\n",
    vga_disp[vga_num].var.xres,
    vga_disp[vga_num].var.yres,
    vga_disp[vga_num].var.bits_per_pixel,
    phys_size[vga_num],
    phys_vmembase[vga_num],
    phys_vgaiobase[vga_num]);
  do_install_cmap(0);

  Iprint(
    " %s FB device %d, using %ldK of video memory @ $%08lX IO @ $%08lX\n",
    fb_info[vga_num].modename,
    vga_num,
    virt_size[vga_num] >> 10,
    virt_vmembase[vga_num],
    virt_vgaiobase[vga_num]);

  return(0);
}
/*endproc SVGA_fb_init() */

/*============================================================================*/


/*--------------------------------------------------------------------------*/

static int 
SVGAfb_switch(int con)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGAfb_switch(%d) \n", con);
  /* Do we have to save the colormap? */
  if (fb_display[currcon].cmap.len)
  {
    fb_get_cmap(&fb_display[currcon].cmap,
      &fb_display[currcon].var,
      1,
      fbhws_ptr->getcolreg);
  }
  /*endif*/
  do_fb_set_var(&fb_display[con].var, 1);
  currcon = con;
  /* Install new colormap */
  do_install_cmap(con);
  return(0);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Update the `var' structure (called by fbcon.c)
    *
    *    This call looks only at yoffset and the FB_VMODE_YWRAP flag in `var'.
    *    Since it's called by a kernel driver, no range checking is done.
    */
/*--------------------------------------------------------------------------*/

static int 
SVGAfb_updatevar(int con)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGAfb_updatevar() \n");
  return(0);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Blank the display.
    */
/*--------------------------------------------------------------------------*/

static void 
SVGAfb_blank(int blank)

/*--------------------------------------------------------------------------*/
{
  /*--------------------------------------------------------------------------*/
  CFuncId("SVGAfb_blank() \n");
  fbhws_ptr->blank(blank);
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Set the colormap
    */
/*--------------------------------------------------------------------------*/

static int 
SVGAfb_setcmap(struct fb_cmap *cmap, int con)

/*--------------------------------------------------------------------------*/
{

  /*--------------------------------------------------------------------------*/
  CFuncId("SVGAfb_setcmap() \n");
  return(M64_op_set_cmap(cmap, 1, con));
}
/*endproc*/

/*============================================================================*/


/*--------------------------------------------------------------------------*/
   /*
    *    Get a Video Mode
    */
/*--------------------------------------------------------------------------*/

static int 
get_video_mode(const char * name)

/*--------------------------------------------------------------------------*/
{
  int i;

  /*--------------------------------------------------------------------------*/
  CFuncId("get_video_mode(%s) \n", name);

  for (i = 1; i < NUM_PREDEF_MODES; i++)
  {
    if (!strcmp(name, svga_fb_modenames[i]))  return(i);
  }
  /*next i*/
  return(0);
}
/*endproc*/

/*============================================================================*/


/*============================================================================*/
/* RCS-ID */

  static char
  rcs_id_c[] =
    "@(#) Linux-m68k: 1997 06 23 19:02:32 BHA: svgafb.c ";

  static char
  rcs_id_h[] = SVGAFB_H_ID;

/*============================================================================*/
